#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Jul 25 16:07:14 2023

@author: kkkk
"""

import pandas as pd
import matplotlib.pyplot as plt
from matplotlib.font_manager import FontProperties

# 读取Excel文件
file_path = '/Users/kkkk/Desktop/B站科普视频的封面研究/科普视频数据表格/数据-最终.xlsx'
df = pd.read_excel(file_path)

# 计算色彩丰富度指数new列的上下四分位数
q1_color_index = df['色彩丰富度指数new'].quantile(0.25)
q3_color_index = df['色彩丰富度指数new'].quantile(0.75)

# 计算色彩丰富度指数new列的四分位距
iqr_color_index = q3_color_index - q1_color_index

# 定义色彩丰富度指数new列的异常值的上下阈值
upper_threshold_color_index = q3_color_index + 1.5 * iqr_color_index
lower_threshold_color_index = q1_color_index - 1.5 * iqr_color_index

# 计算实际播放量列的上下四分位数
q1_play_count = df['实际播放量'].quantile(0.25)
q3_play_count = df['实际播放量'].quantile(0.75)

# 计算实际播放量列的四分位距
iqr_play_count = q3_play_count - q1_play_count

# 定义实际播放量列的异常值的上下阈值
upper_threshold_play_count = q3_play_count + 1.5 * iqr_play_count
lower_threshold_play_count = q1_play_count - 1.5 * iqr_play_count

# 去除色彩丰富度指数new和实际播放量列的异常值
df_filtered = df[
    (df['色彩丰富度指数new'] >= lower_threshold_color_index) & (df['色彩丰富度指数new'] <= upper_threshold_color_index) &
    (df['实际播放量'] >= lower_threshold_play_count) & (df['实际播放量'] <= upper_threshold_play_count)
]

# 提取过滤后的数据
x_filtered = df_filtered['色彩丰富度指数new']
y_filtered = df_filtered['实际播放量']

# 定义亮度的等宽区间数量
num_bins = 6

# 计算每个区间的宽度
bin_width = (x_filtered.max() - x_filtered.min()) / num_bins

# 使用 pd.cut 函数进行等宽分组
x_bins = pd.cut(x_filtered, bins=pd.interval_range(start=x_filtered.min(), end=x_filtered.max() + bin_width, freq=bin_width))

# 计算亮度区间的均播放量
avg_play_count = y_filtered.groupby(x_bins).mean()

# 计算亮度区间的播放量标准差
std_per_interval = y_filtered.groupby(x_bins).std()

# 转换为DataFrame
df_avg_play_count = pd.DataFrame({'均播放量': avg_play_count.values}, index=avg_play_count.index)

# 设置中文字体
font_path = '/Users/kkkk/Desktop/STHeitiLight.ttc'
font_prop = FontProperties(fname=font_path)

# 提取区间的左右边界作为x轴标签
x_labels = [f'{interval.left:.2f}-{interval.right:.2f}' for interval in df_avg_play_count.index]

# 绘制折线图带误差线
std_multiplier = 0.1  # 标准差倍数
y_err = std_per_interval * std_multiplier  # 计算标准差倍数的误差范围

plt.errorbar(x_labels, df_avg_play_count['均播放量'], yerr=y_err, fmt='o-', color='blue', capsize=4)

# 添加标题和轴标签
plt.title('Average Play Count in Color richness Intervals', fontproperties=font_prop)
plt.xlabel('Color richness range', fontproperties=font_prop)
plt.ylabel('Average play', fontproperties=font_prop)

# 设置网格线
plt.grid(axis='y', linestyle='--')

# 保存图像为PNG文件
output_file = '/Users/kkkk/Desktop/色彩区间均播放量折线图-标准差.png'
plt.savefig(output_file, dpi=300, bbox_inches='tight')

# 显示图像
plt.show()